// Copyright (c) 2015, Electric Cloud, Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   - Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//
//   - Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// (As an historical note, the above license instantiates
// the "BSD 2-Clause License" template that was presented
// by the Open Source Initiative at
//
//   http://www.opensource.org/licenses/bsd-license.php
//
// on 2012-02-23.)

// Summary:
//
// This file implements a two basic tests: that you can include
// various wrapper headers and still be able to compile, and that
// the offsets are correct where an interface extends more than
// one other interface.
//
// The order of these includes is basically arbitrary, though at
// the moment we try to go from core includes to specific ones,
// just to simplify typical compiler errors, should they arise.

#include <vrpsx_allocator.hpp>
#include <vrpsx_reference_counted.hpp>
#include <vrpsx_argv.hpp>
#include <vrpsx_identified.hpp>
#include <vrpsx_system_error.hpp>
#include <vrpsx_single_owner.hpp>
#include <vrpsx_system.hpp>
#include <vrpsx_embedding.hpp>
#include <stdio.h>
#include <stdlib.h>

using namespace vrpsx;
using namespace std;


template< typename Derived, typename Base >
unsigned baseOffset()
{
    Derived *derived = (Derived *)0 + 1;
    Base *base = derived;
    return (unsigned)(vrpsx_uintptr_t)((char *)base - (char *)derived);
}

int main(int argc, char **argv)
{
    int exit_code = EXIT_SUCCESS;

    unsigned actual = baseOffset< system_t, allocator_t >();
    unsigned expected = offsetof(vrpsx_system_t, allocator);
    if (actual != expected) {
        printf("%s:%d: wrong offset of allocator in system: %u != %u\n",
                __FILE__, __LINE__, actual, expected);
        exit_code = EXIT_FAILURE;
    }

    actual = baseOffset< embedding_t, system_t >();
    expected = offsetof(vrpsx_embedding_t, system);
    if (actual != expected) {
        printf("%s:%d: wrong offset of system in embedding: %u != %u\n",
                __FILE__, __LINE__, actual, expected);
        exit_code = EXIT_FAILURE;
    }

    actual = baseOffset< embedding_t, allocator_t >();
    expected = offsetof(vrpsx_embedding_t, system.allocator);
    if (actual != expected) {
        printf("%s:%d: wrong offset of allocator in embedding: %u != %u\n",
                __FILE__, __LINE__, actual, expected);
        exit_code = EXIT_FAILURE;
    }

    return exit_code;
}
