/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * An interface to the initial features of a virtual POSIX-like system.
 */

#ifndef VRPSX_SYSTEM_INITIAL_H
#define VRPSX_SYSTEM_INITIAL_H 1

#include "vrpsx_system.h"
#include "vrpsx_argv.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* An interface to the initial features of a virtual POSIX-like system
 * that one would typically make available to a main-like function.
 *
 * You can a pointer to cast this interface to (vrpsx_identified_t *)
 * because this interface extends vrpsx_identified_t.
 *
 * Extends: vrpsx_identified_t.
 * Interface Lifetime: implied by context.
 * Thread Safety: requires synchronization.
 */
typedef struct vrpsx_system_initial_struct {
#define VRPSX_SYSTEM_INITIAL_MEMBERS(VTBL, SYSTEM)                      \
    VRPSX_IDENTIFIED_MEMBERS(VTBL)                                      \
                                                                        \
    /* Virtual command-line arguments.  This pointer owns one           \
     * reference count.  You may take ownership of that reference       \
     * count by copying the pointer value to another location and       \
     * then overwriting this member with NULL.                          \
     *                                                                  \
     * NOTE: The encoding is chosen by the parent process.              \
     */                                                                 \
    vrpsx_argv_t *arguments;                                            \
                                                                        \
    /* Virtual environment variables.  This pointer owns one            \
     * reference count.  You may take ownership of that reference       \
     * count by copying the pointer value to another location and       \
     * then overwriting this member with NULL.                          \
     *                                                                  \
     * NOTE: The encoding is chosen by the parent process.              \
     */                                                                 \
    vrpsx_argv_t *environment;                                          \
                                                                        \
    /* The virtual POSIX-like system itself.  This pointer owns one     \
     * reference count.  You may take ownership of that reference       \
     * count by copying the pointer value to another location and       \
     * then overwriting this member with NULL.                          \
     */                                                                 \
    SYSTEM *system;

    VRPSX_SYSTEM_INITIAL_MEMBERS(
        struct vrpsx_system_initial_vtbl_struct,
        vrpsx_system_t)
} vrpsx_system_initial_t;

typedef struct vrpsx_system_initial_end_struct {
} vrpsx_system_initial_end_t;

typedef struct vrpsx_system_initial_vtbl_struct {
#define VRPSX_SYSTEM_INITIAL_VTBL_MEMBERS(TYPE)                         \
    VRPSX_IDENTIFIED_VTBL_MEMBERS(TYPE)                                 \
                                                                        \
    /* End marker for documentation and to help detect initialization bugs. \
     */                                                                 \
    vrpsx_system_initial_end_t const *system_initial_end;

    VRPSX_SYSTEM_INITIAL_VTBL_MEMBERS(vrpsx_system_initial_t);
} vrpsx_system_initial_vtbl_t;


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_SYSTEM_INITIAL_H */
